function varargout = getfile_list(varargin)
% Gets filelist for QuikSCAT L3 (Mapped) winds between two input dates.
% from an html page, in this case
% http://dods.jpl.nasa.gov/opendap/ocean_wind/quikscat/L3/data/1999/content
% s.html
%
% This file replaces the previously-existing "ftp-dependent" software. This
% is because some systems were experiencing problems related to ftp and the
% SeaWinds interface. Resolution: seems that Mac systems had most
% difficulty with ftp problem, so this file (and associated get_seawinds
% Version 2.1.1 should work well on the Mac -- tested on both PPC and
% Intel).
%
% filename = getfile_list(URLbase,date1,date2,daystride);
%
% INPUT(S):
%
%   URLbase -- see below for example
%   date1 -- 3 or 6 element gregorian calendar vector [yr mo da] or
%            [yr mo da hr mi se], start date
%   date2 -- 3 or 6 element gregorian calendar vector [yr mo da] or
%            [yr mo da hr mi se], stop date
%   daystride -- (optional) default = 1, integer defining resolution of
%            time for the datafiles returned. For example,
%               daystride = 1 --> every file
%               daystride = 2 --> every other file
%
% OUTPUT(S):
%
%   filename -- list of filenames in a cell array
%   tmat -- (optional) time (matlab time) of each data file
%   status -- (optional) 0 if error, 1 if success (reading url address)
%
% EXAMPLE:
%
% To obtain a list of files between July 19, 1999 and July 21, 1999
%
%         URLbase =
%         'http://dods.jpl.nasa.gov/opendap/ocean_wind/quikscat/L3/data';
%         date1 = [1999 07 19];
%         date2 = [1999 07 21];
%         daystride = 1;
%         [filename,tmat] = getfile_list(URLbase,date1,date2,daystride);
%
% To obtain the list of files between July 19, 1999 and the present
%
%         URLbase =
%         'http://dods.jpl.nasa.gov/opendap/ocean_wind/quikscat/L3/data';
%         date1 = [1999 07 19];
%         date2 = datevec(now);
%         daystride = 1;
%         [filename,tmat] = getfile_list(URLbase,date1,date2,daystride);
%
% OPeNDAP Science Team
% Copyright 2007, 2008
% $Version 2.0.1$

%==========================================================================
% Christian Buckingham
%
% REVISION HISTORY:
% 2007/09/01 1.0.0 created, ceb
% 2008/03/01 2.0.0 version number updated for release, ceb
% 2008/03/18 2.0.1 revamped to read html code in place of ftp, ceb
%
% Uses third-party software:
% greg2.m -- part of "Air-Sea Interaction" Toolbox at
%              http://woodshole.er.usgs.gov/operations/sea-mat/air_sea-html
%              /yearday.m
%==========================================================================

% Parse inputs.
if nargin == 3
    URLbase = varargin{1};
    date1 = varargin{2};
    date2 = varargin{3};
    daystride = 1;
elseif nargin == 4
    URLbase = varargin{1};
    date1 = varargin{2};
    date2 = varargin{3};
    daystride = varargin{4};
else
    error('Incorrect number of input arguments.')
end

% Define keyword.
kyword = 'QS_XWGRD3';

% Construct url address.
slash = '/';
%URLbase = 'http://dods.jpl.nasa.gov/opendap/ocean_wind/quikscat/L3/data';
URLpage = 'contents.html';

% Define (matlab time) days of interest.
date1_mtime = datenum(date1);
date2_mtime = datenum(date2);

% Generate string years of interest.
yearofint = (date1(1):date2(1));
nyrs = length(yearofint);
if nyrs>100000
    error('Incorrect number or years');
end
yyyy = cell(1,nyrs);
for ii = 1:nyrs
    yyyy{ii} = num2str(yearofint(ii));
end

% Read web page contents.
status = 1;
filename = {};
%tt = {};
tmat = [];
kk = 1;
for ii = 1:nyrs
    
    URLyear = yyyy{ii};
    url = strcat(URLbase,slash,URLyear,slash,URLpage);
    [s,status] = urlread(url);
    if status == 0
        if narout == 1
            varargout{1} = filename;
        elseif nargout == 2
            varargout{1} = filename;
            varargout{2} = tmat;
        elseif nargout == 3
            varargout{1} = filename;
            varargout{2} = tmat;
            varargout{3} = status;
        end
        return
    end
    
    istr = strfind(s,kyword);
    itmp = (2:7:length(istr));
    istr_important = istr(itmp);
    for jj = 1:length(istr_important)
        
        itmp = istr_important(jj);
        tmp_filename = s(itmp:itmp+31);
        
        % Determine if in bounds of date1 and date2.
        iidx = strfind(tmp_filename,kyword)+length(kyword)+1;
        tmp_year = str2double(tmp_filename(iidx:iidx+3));
        tmp_yearday = str2double(tmp_filename(iidx+4:iidx+6));
        tmp_gtime = op_greg2(tmp_yearday-1,tmp_year); %GREG2 takes 0.0 = Jan1
        tmp_mtime = datenum(tmp_gtime);
        
%         yr = str2double(tmp_filename(11:14));
%         yd = str2double(tmp_filename(15:17));
%         tmp_gtime = op_greg2(yd-1,yr);
%         tmp_mtime = datenum(tmp_gtime);
        
        if tmp_mtime>=date1_mtime && tmp_mtime<=date2_mtime
            
            filename{kk} = strcat(URLbase,slash,URLyear,slash,tmp_filename);
            %tt{kk} = tmp_gtime;
            tmat(kk) = tmp_mtime;
            kk = kk + 1;
            
        end%%datafile between bounds of date1 and date2.
        
    end
    
end

nfile_total = length(filename);
file = filename;
clear filename

filename = {};
kk = 1;
for ifile = 1:daystride:nfile_total
    filename{kk} = file{ifile};
    kk = kk + 1;
end

% Save to .mat file.
if (0)
save gui_seawinds_filelist.mat filename tmat
end

% Define output arguments.
if nargout == 1
    varargout{1} = filename;
elseif nargout == 2
    varargout{1} = filename;
    varargout{2} = tmat;
elseif nargout == 3
    varargout{1} = filename;
    varargout{2} = tmat;
    varargout{3} = status;
else
    error('Number of output arguments incorrect')
end

return %endoffunction

%==========================================================================
% PREVIOUS FTP-DEPENDENT CODE.
%==========================================================================
% function filename = get_file_list(varargin)
% % Gets filelist for QuikSCAT L3 (Mapped) winds between two input dates.
% %
% % filename = get_file_list(toplevel,date1,date2,daystride) -- local
% % filename = get_file_list(ftphost,toplevel,date1,date2,daystride) -- ftp
% %
% % Works for both locally-located data and FTP site.
% %
% % If used for locally-located data, such as on a hard-dist or network
% % drive, it assumes the data are in a format similar to NASA's
% % Physical Oceanography Distributed Active Archive Center.
% %
% % INPUT(S):
% %
% %   ftphost -- (optional) host of ftp site on which data resides
% %            For example, data located at NASA's PO.DAAC have
% %            an ftp host of 'podaac.jpl.nasa.gov'.
% %            If an ftphost is unspecified, the software assumes a local
% %            drive.
% %   toplevel -- main directory (top-level) on which the
% %            "1999","2000","2001",etc. folders reside. For example, on
% %            at PO.DAAC's ftpsite directories "1999","2000","2001",etc.
% %            are located at '/pub/ocean_wind/quikscat/L3/data/' or
% %            'ftp://podaac.jpl.nasa.gov/pub/ocean_wind/quikscat/L3/data/'.
% %   date1 -- 3 or 6 element gregorian calendar vector [yr mo da] or
% %            [yr mo da hr mi se], start date
% %   date2 -- 3 or 6 element gregorian calendar vector [yr mo da] or
% %            [yr mo da hr mi se], stop date
% %   daystride -- (optional) default = 1, integer defining resolution of
% %            time for the datafiles returned. For example,
% %               daystride = 1 --> every file
% %               daystride = 2 --> every other file
% %               ...
% %
% % OUTPUT(S):
% %
% %   filename -- list of filenames in [toplevel] directory.
% %
% % EXAMPLE:
% %
% %   For data located on a local disk ...
% %
% %         toplevel = 'D:\quikscat';
% %         date1 = [1999 07 20];
% %         date2 = [2000 01 07];
% %         daystride = 1;
% %         filename1 = get_file_list(toplevel,date1,date2,daystride);
% %
% %   For data located on an ftp: site (remotely-located disk with ftp
% %   access) ...
% %
% %         ftphost = 'podaac.jpl.nasa.gov';
% %         toplevel = '/pub/ocean_wind/quikscat/L3/data/';
% %         date1 = [1999 07 20];
% %         date2 = [2000 01 07];
% %         daystride = 1;
% %         filename2 = get_file_list(ftphost,toplevel,date1,date2,daystride);
% %
% % University of Rhode Island
% % September 2007
% % $Version 1.00$
% 
% % Christian Buckingham
% % September 2007
% %
% % Modification History:
% %
% % Uses third-party software:
% % greg2.m -- part of "Air-Sea Interaction" Toolbox at
% %              http://woodshole.er.usgs.gov/operations/sea-mat/air_sea-html
% %              /yearday.m
% %==========================================================================
% 
% opendap = 1;
% 
% % PARSE INPUT(S).
% if nargin == 5
%     ftphost = varargin{1};
%     toplevel = varargin{2};
%     date1 = varargin{3};
%     date2 = varargin{4};
%     daystride = varargin{5};
%     
% elseif nargin == 4
%     toplevel = varargin{1};
%     date1 = varargin{2};
%     date2 = varargin{3};
%     daystride = varargin{4};
% else
%     error('Incorrect number of input arguments.')
% end
% 
% % PRELIMINARIES.
% if nargin == 4
%     dummy = toplevel(end);
%     if strcmp(dummy,'/') || strcmp(dummy,'\')
%         toplevel = toplevel(1:end-1);
%     end
%     slash = get_slash;
% end
% %daystride = round(daystride); %ROUND TO INTEGER.
% %daystride = 1; %defines "every file" (daystride = 1),
% %               %  "every other file" (daystride = 2)
% kyword = 'QS_XWGRD3';
% 
% % FTP OR LOCAL?
% if nargin == 5
% if strcmp(toplevel(1:3),'ftp')
%     ftphost = strrep(toplevel,'ftp://','');
%     [ftphost,toplevel] = strtok(ftphost,'/');
% end
% slash = '/';
% end
% %==========================================================================
% %==========================================================================
% % OPENDAP DIRECTORY.
% opendap_dir = strcat('http://dods.jpl.nasa.gov/opendap/',...
%               'ocean_wind/quikscat/L3/data');
% 
% % YEARS OF INTEREST.
% yearofint = (date1(1):date2(1));
% nyrs = length(yearofint);
% yyyy = cell(1,nyrs);
% for ii = 1:nyrs
%     yyyy{ii} = num2str(yearofint(ii));
% end
% 
% % DAYS OF INTEREST.
% date1_mtime = datenum(date1);
% date2_mtime = datenum(date2);
% 
% % FTP port.
% if (nargin == 5)
%     f=ftp(ftphost,'anonymous','');
% end
% 
% % LOOK IN DIRECTORY STRUCTURE FOR FILES.
% filename = {};
% KK = 1;
% for iyr = 1:nyrs
%     subdir = yyyy{iyr};
%     if (nargin == 5)
%         file = dir(f,strcat(toplevel,slash,subdir,slash,kyword,'*gz'));
%     else
%         file = dir(strcat(toplevel,slash,subdir,slash,kyword,'*gz'));
%     end
%     nfile = length(file);
%     kk = 1;
%     tmp_filename = {};
%     for ii = 1:nfile
%         tmp = file(ii).name;
%         iidx = strfind(tmp,kyword)+length(kyword)+1;
%         tmp_year = str2double(tmp(iidx:iidx+3));
%         tmp_yearday = str2double(tmp(iidx+4:iidx+6));
%         tmp_gtime = op_greg2(tmp_yearday-1,tmp_year); %GREG2 takes 0.0 = Jan1
%         tmp_mtime = datenum(tmp_gtime);
%         if ~file(ii).isdir 
%             if tmp_mtime>=date1_mtime && tmp_mtime<=date2_mtime
%             tmp_filename{kk} = file(ii).name;
%             %filename{KK} = file(ii).name; %no pathname
%             if ~opendap
%                 filename{KK} = strcat(toplevel,slash,subdir,slash,...
%                 file(ii).name); %full pathname
%             elseif opendap
%                 filename{KK} = strcat(opendap_dir,slash,subdir,slash,...
%                 file(ii).name); %http:// address for opendap connection
%             end
%             kk = kk + 1;
%             KK = KK+1;
%             end%%datafile between bounds of date1 and date2
%         end
%     end%ii
%     %nfile_this_directory = length(tmp_filename);
% end
% nfile_total = length(filename);
% file = filename;
% clear filename
% 
% filename = {};
% kk = 1;
% for ifile = 1:daystride:nfile_total
%     filename{kk} = file{ifile};
%     kk = kk + 1;
% end
% 
% % FTP port.
% if (nargin == 5)
%     close(f);
% end
% 
% return %endoffunction
% 
% %==========================================================================
% function slash = get_slash
% 
% slash = '';
% if (ispc), slash='\'; end
% if (isunix), slash='/'; end
% if (ismac), slash='/'; end
% %==========================================================================
% function aa = ismac
% 
% dum = computer;
% aa = strcmpi(dum,'mac') || strcmpi(dum,'maci');
% %==========================================================================